from django.test import TestCase
from django.urls import reverse

from app_utils.testdata_factories import UserMainFactory


class TestHooks(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.user = UserMainFactory()

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.html_menu = f"""
            <a class="nav-link flex-fill align-self-center me-auto active" href="{reverse('aa_contacts:index')}">
                Contacts
            </a>
        """

    def test_menu_hook(self):
        self.client.force_login(self.user)
        response = self.client.get(reverse('aa_contacts:index'))
        self.assertContains(response, self.html_menu, html=True)



# Generated by Django 4.2.13 on 2024-05-29 16:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('aa_contacts', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='general',
            options={'default_permissions': (), 'managed': False, 'permissions': (('manage_alliance_contacts', 'Can manage alliance contacts'), ('manage_corporation_contacts', 'Can manage corporation contacts'), ('view_alliance_notes', 'Can view notes on alliance contacts'), ('view_corporation_notes', 'Can view notes on corporation contacts'))},
        ),
        migrations.AlterField(
            model_name='alliancecontact',
            name='labels',
            field=models.ManyToManyField(blank=True, related_name='contacts', to='aa_contacts.alliancecontactlabel'),
        ),
        migrations.CreateModel(
            name='CorporationContactLabel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label_id', models.BigIntegerField()),
                ('label_name', models.CharField(max_length=255)),
                ('corporation', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='contact_labels', to='eveonline.evecorporationinfo')),
            ],
            options={
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='CorporationContact',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contact_id', models.BigIntegerField()),
                ('contact_type', models.CharField(choices=[('character', 'Character'), ('corporation', 'Corporation'), ('alliance', 'Alliance'), ('faction', 'Faction')], max_length=11)),
                ('standing', models.FloatField()),
                ('notes', models.TextField(blank=True, default='')),
                ('corporation', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='contacts', to='eveonline.evecorporationinfo')),
                ('labels', models.ManyToManyField(blank=True, related_name='contacts', to='aa_contacts.corporationcontactlabel')),
            ],
            options={
                'default_permissions': (),
            },
        ),
    ]



from django.apps import apps
from django.contrib import admin

from .models import AllianceContact, CorporationContact, StandingFilter


@admin.register(AllianceContact)
class AllianceContactAdmin(admin.ModelAdmin):
    exclude = ('contact_id', )
    readonly_fields = ('alliance', 'contact_type', 'standing', 'labels', )


@admin.register(CorporationContact)
class CorporationContactAdmin(admin.ModelAdmin):
    exclude = ('contact_id', )
    readonly_fields = ('corporation', 'contact_type', 'standing', 'labels', )


class StandingFilterAdmin(admin.ModelAdmin):
    raw_id_fields = ('corporations', 'alliances', )


if apps.is_installed('securegroups'):
    admin.site.register(StandingFilter, StandingFilterAdmin)



from django.test import TestCase
from django.contrib.auth.models import User

from app_utils.testdata_factories import UserMainFactory, EveCorporationInfoFactory, EveAllianceInfoFactory, EveCharacterFactory
from app_utils.testing import add_character_to_user

from aa_contacts.models import StandingFilter, AllianceContact, CorporationContact


class TestStandingFilter(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.corp_red = EveCorporationInfoFactory()
        cls.corp_blue = EveCorporationInfoFactory()
        cls.alliance_red = EveAllianceInfoFactory()
        cls.alliance_blue = EveAllianceInfoFactory()
        corp_in_red = EveCorporationInfoFactory(alliance=cls.alliance_red)
        corp_in_blue = EveCorporationInfoFactory(alliance=cls.alliance_blue)

        cls.char_red_corp = EveCharacterFactory(corporation=cls.corp_red)
        cls.char_blue_corp = EveCharacterFactory(corporation=cls.corp_blue)
        cls.char_red_alliance = EveCharacterFactory(corporation=corp_in_red)
        cls.char_blue_alliance = EveCharacterFactory(corporation=corp_in_blue)

        cls.user = UserMainFactory()

        cls.corp_user = cls.user.profile.main_character.corporation
        cls.alliance_user = cls.user.profile.main_character.alliance

        AllianceContact.objects.bulk_create([
            AllianceContact(
                alliance=cls.alliance_user,
                contact_id=cls.alliance_red.alliance_id,
                contact_type=AllianceContact.ContactTypeOptions.ALLIANCE,
                standing=-10.0,
            ),
            AllianceContact(
                alliance=cls.alliance_user,
                contact_id=cls.alliance_blue.alliance_id,
                contact_type=AllianceContact.ContactTypeOptions.ALLIANCE,
                standing=10.0,
            ),
        ])

        CorporationContact.objects.bulk_create([
            CorporationContact(
                corporation=cls.corp_user,
                contact_id=cls.corp_red.corporation_id,
                contact_type=CorporationContact.ContactTypeOptions.CORPORATION,
                standing=-10.0,
            ),
            CorporationContact(
                corporation=cls.corp_user,
                contact_id=cls.corp_blue.corporation_id,
                contact_type=CorporationContact.ContactTypeOptions.CORPORATION,
                standing=10.0,
            ),
        ])

    def test_str(self):
        filter = StandingFilter.objects.create(
            name='Test Filter',
            description='Test Description',
            comparison=StandingFilter.ComparisonOptions.GREATER_THAN,
            standing=0.0,
        )
        self.assertEqual(str(filter), 'Test Filter: Test Description')

    def test_audit_filter_at_least_one_all_chars(self):
        filter = StandingFilter.objects.create(
            name='Test Filter',
            description='Test Description',
            comparison=StandingFilter.ComparisonOptions.GREATER_THAN,
            standing=0.0,
            check_type=StandingFilter.CheckTypeOptions.AT_LEAST_ONE_CHARACTER,
        )
        filter.alliances.add(self.alliance_user)

        filter_red = StandingFilter.objects.create(
            name='Test Filter Red',
            description='Test Description Red',
            comparison=StandingFilter.ComparisonOptions.LESS_THAN,
            standing=-5.0,
            check_type=StandingFilter.CheckTypeOptions.AT_LEAST_ONE_CHARACTER,
        )
        filter_red.alliances.add(self.alliance_user)

        user_blue_and_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_blue_and_red, self.char_blue_alliance)

        self.assertTrue(filter.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        self.assertTrue(filter_red.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        user_blue = UserMainFactory(main_character__character=self.char_blue_alliance)

        self.assertTrue(filter.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

        self.assertFalse(filter_red.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

    def test_audit_filter_at_least_one_only_main(self):
        filter = StandingFilter.objects.create(
            name='Test Filter',
            description='Test Description',
            comparison=StandingFilter.ComparisonOptions.GREATER_THAN,
            standing=0.0,
            check_type=StandingFilter.CheckTypeOptions.AT_LEAST_ONE_CHARACTER,
            only_main=True,
        )
        filter.alliances.add(self.alliance_user)

        filter_red = StandingFilter.objects.create(
            name='Test Filter Red',
            description='Test Description Red',
            comparison=StandingFilter.ComparisonOptions.LESS_THAN,
            standing=-5.0,
            check_type=StandingFilter.CheckTypeOptions.AT_LEAST_ONE_CHARACTER,
            only_main=True,
        )
        filter_red.alliances.add(self.alliance_user)

        user_blue_and_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_blue_and_red, self.char_blue_alliance)

        self.assertFalse(filter.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        self.assertTrue(filter_red.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

    def test_audit_filter_all_characters_all_chars(self):
        filter = StandingFilter.objects.create(
            name='Test Filter',
            description='Test Description',
            comparison=StandingFilter.ComparisonOptions.GREATER_THAN,
            standing=0.0,
            check_type=StandingFilter.CheckTypeOptions.ALL_CHARACTERS,
        )
        filter.alliances.add(self.alliance_user)
        filter.corporations.add(self.corp_user)

        filter_red = StandingFilter.objects.create(
            name='Test Filter Red',
            description='Test Description Red',
            comparison=StandingFilter.ComparisonOptions.LESS_THAN,
            standing=-5.0,
            check_type=StandingFilter.CheckTypeOptions.ALL_CHARACTERS,
        )
        filter_red.alliances.add(self.alliance_user)
        filter_red.corporations.add(self.corp_user)

        user_blue_and_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_blue_and_red, self.char_blue_alliance)

        self.assertFalse(filter.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        self.assertFalse(filter_red.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        user_blue = UserMainFactory(main_character__character=self.char_blue_alliance)
        add_character_to_user(user_blue, self.char_blue_corp)

        self.assertTrue(filter.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

        self.assertFalse(filter_red.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

        user_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_red, self.char_red_corp)

        self.assertFalse(filter.audit_filter(
            User.objects.filter(pk=user_red.pk)
        )[user_red.pk]['check'])

        self.assertTrue(filter_red.audit_filter(
            User.objects.filter(pk=user_red.pk)
        )[user_red.pk]['check'])

    def test_audit_filter_all_characters_only_main(self):
        filter = StandingFilter.objects.create(
            name='Test Filter',
            description='Test Description',
            comparison=StandingFilter.ComparisonOptions.GREATER_THAN,
            standing=0.0,
            check_type=StandingFilter.CheckTypeOptions.ALL_CHARACTERS,
            only_main=True,
        )
        filter.alliances.add(self.alliance_user)
        filter.corporations.add(self.corp_user)

        filter_red = StandingFilter.objects.create(
            name='Test Filter Red',
            description='Test Description Red',
            comparison=StandingFilter.ComparisonOptions.LESS_THAN,
            standing=-5.0,
            check_type=StandingFilter.CheckTypeOptions.ALL_CHARACTERS,
            only_main=True,
        )
        filter_red.alliances.add(self.alliance_user)
        filter_red.corporations.add(self.corp_user)

        user_blue_and_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_blue_and_red, self.char_blue_alliance)

        self.assertFalse(filter.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        self.assertTrue(filter_red.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        user_blue = UserMainFactory(main_character__character=self.char_blue_alliance)
        add_character_to_user(user_blue, self.char_red_corp)

        self.assertTrue(filter.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

        self.assertFalse(filter_red.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

    def test_audit_filter_no_character_all_chars(self):
        filter = StandingFilter.objects.create(
            name='Test Filter',
            description='Test Description',
            comparison=StandingFilter.ComparisonOptions.GREATER_THAN,
            standing=0.0,
            check_type=StandingFilter.CheckTypeOptions.NO_CHARACTER,
        )
        filter.alliances.add(self.alliance_user)
        filter.corporations.add(self.corp_user)

        filter_red = StandingFilter.objects.create(
            name='Test Filter Red',
            description='Test Description Red',
            comparison=StandingFilter.ComparisonOptions.LESS_THAN,
            standing=-5.0,
            check_type=StandingFilter.CheckTypeOptions.NO_CHARACTER,
        )
        filter_red.alliances.add(self.alliance_user)
        filter_red.corporations.add(self.corp_user)

        user_blue_and_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_blue_and_red, self.char_blue_alliance)

        self.assertFalse(filter.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        self.assertFalse(filter_red.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        user_blue = UserMainFactory(main_character__character=self.char_blue_alliance)
        add_character_to_user(user_blue, self.char_blue_corp)

        self.assertFalse(filter.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

        self.assertTrue(filter_red.audit_filter(
            User.objects.filter(pk=user_blue.pk)
        )[user_blue.pk]['check'])

        user_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_red, self.char_red_corp)

        self.assertTrue(filter.audit_filter(
            User.objects.filter(pk=user_red.pk)
        )[user_red.pk]['check'])

        self.assertFalse(filter_red.audit_filter(
            User.objects.filter(pk=user_red.pk)
        )[user_red.pk]['check'])

    def test_audit_filter_no_characters_only_main(self):
        filter = StandingFilter.objects.create(
            name='Test Filter',
            description='Test Description',
            comparison=StandingFilter.ComparisonOptions.GREATER_THAN,
            standing=0.0,
            check_type=StandingFilter.CheckTypeOptions.NO_CHARACTER,
            only_main=True,
        )
        filter.alliances.add(self.alliance_user)
        filter.corporations.add(self.corp_user)

        filter_red = StandingFilter.objects.create(
            name='Test Filter Red',
            description='Test Description Red',
            comparison=StandingFilter.ComparisonOptions.LESS_THAN,
            standing=-5.0,
            check_type=StandingFilter.CheckTypeOptions.NO_CHARACTER,
            only_main=True,
        )
        filter_red.alliances.add(self.alliance_user)
        filter_red.corporations.add(self.corp_user)

        user_blue_and_red = UserMainFactory(main_character__character=self.char_red_alliance)
        add_character_to_user(user_blue_and_red, self.char_blue_alliance)

        self.assertTrue(filter.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])

        self.assertFalse(filter_red.audit_filter(
            User.objects.filter(pk=user_blue_and_red.pk)
        )[user_blue_and_red.pk]['check'])



from celery import shared_task, group
from celery_once import QueueOnce

from django.db import transaction
from django.utils import timezone

from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger

from .models import AllianceContact, AllianceContactLabel, AllianceToken, CorporationToken, CorporationContact, CorporationContactLabel
from .provider import esi

logger = get_extension_logger(__name__)


@shared_task(base=QueueOnce, once={'graceful': True})
def load_alliance_contact_name(contact_pk: int):
    contact = AllianceContact.objects.get(pk=contact_pk)
    contact.contact_name


@shared_task(base=QueueOnce, once={'graceful': True})
def load_corporation_contact_name(contact_pk: int):
    contact = CorporationContact.objects.get(pk=contact_pk)
    contact.contact_name


@shared_task(base=QueueOnce, once={'graceful': True})
def update_alliance_contacts(alliance_id: int):
    contacts_to_load = []

    try:
        alliance = EveAllianceInfo.objects.get(alliance_id=alliance_id)
    except EveAllianceInfo.DoesNotExist:
        alliance = EveAllianceInfo.objects.create_alliance(alliance_id)

    try:
        alliance_token = AllianceToken.objects.with_valid_tokens().select_related('token').get(alliance=alliance)
    except AllianceToken.DoesNotExist:
        raise ValueError(f"No valid token found for alliance {alliance}")

    labels_data = (
        esi.client
        .Contacts
        .get_alliances_alliance_id_contacts_labels(
            alliance_id=alliance_id,
            token=alliance_token.token.valid_access_token()
        )
        .results()
    )

    labels = {label['label_id']: label['label_name'] for label in labels_data}

    contacts_data = (
        esi.client
        .Contacts
        .get_alliances_alliance_id_contacts(
            alliance_id=alliance_id,
            token=alliance_token.token.valid_access_token()
        )
        .results()
    )

    contact_ids = {
        contact['contact_id']: {
            'contact_type': contact['contact_type'],
            'label_ids': contact.get('label_ids', []),
            'standing': contact['standing']
        } for contact in contacts_data
    }

    with transaction.atomic():
        AllianceContactLabel.objects.filter(
            alliance=alliance
        ).exclude(
            label_id__in=labels.keys()
        ).delete()

        for label_id, label_name in labels.items():
            label, _ = AllianceContactLabel.objects.update_or_create(
                alliance=alliance,
                label_id=label_id,
                defaults={'label_name': label_name}
            )

            labels[label_id] = label

            AllianceContact.objects.filter(
                alliance=alliance
            ).exclude(
                contact_id__in=contact_ids.keys()
            ).update(standing=0.0)

        for contact_id, contact_data in contact_ids.items():
            contact, _ = AllianceContact.objects.update_or_create(
                alliance=alliance,
                contact_id=contact_id,
                defaults={
                    'contact_type': contact_data['contact_type'],
                    'standing': contact_data['standing']
                }
            )

            contact.labels.clear()
            if contact_data['label_ids'] is not None:
                contact.labels.set([labels[label_id] for label_id in contact_data['label_ids']])

            contacts_to_load.append(contact.pk)

        alliance_token.last_update = timezone.now()
        alliance_token.save()

    group(load_alliance_contact_name.si(pk) for pk in contacts_to_load).delay()


@shared_task(base=QueueOnce, once={'graceful': True})
def update_corporation_contacts(corporation_id: int):
    contacts_to_load = []

    try:
        corporation = EveCorporationInfo.objects.get(corporation_id=corporation_id)
    except EveCorporationInfo.DoesNotExist:
        corporation = EveCorporationInfo.objects.create_corporation(corporation_id)

    try:
        corporation_token = CorporationToken.objects.with_valid_tokens().select_related('token').get(corporation=corporation)
    except CorporationToken.DoesNotExist:
        raise ValueError(f"No valid token found for alliance {corporation}")

    labels_data = (
        esi.client
        .Contacts
        .get_corporations_corporation_id_contacts_labels(
            corporation_id=corporation_id,
            token=corporation_token.token.valid_access_token()
        )
        .results()
    )

    labels = {label['label_id']: label['label_name'] for label in labels_data}

    contacts_data = (
        esi.client
        .Contacts
        .get_corporations_corporation_id_contacts(
            corporation_id=corporation_id,
            token=corporation_token.token.valid_access_token()
        )
        .results()
    )

    contact_ids = {
        contact['contact_id']: {
            'contact_type': contact['contact_type'],
            'label_ids': contact.get('label_ids', []),
            'standing': contact['standing'],
            'is_watched': contact.get('is_watched', None)
        } for contact in contacts_data
    }

    with transaction.atomic():
        CorporationContactLabel.objects.filter(
            corporation=corporation
        ).exclude(
            label_id__in=labels.keys()
        ).delete()

        for label_id, label_name in labels.items():
            label, _ = CorporationContactLabel.objects.update_or_create(
                corporation=corporation,
                label_id=label_id,
                defaults={'label_name': label_name}
            )

            labels[label_id] = label

            CorporationContact.objects.filter(
                corporation=corporation
            ).exclude(
                contact_id__in=contact_ids.keys()
            ).update(standing=0.0)

        for contact_id, contact_data in contact_ids.items():
            contact, _ = CorporationContact.objects.update_or_create(
                corporation=corporation,
                contact_id=contact_id,
                defaults={
                    'contact_type': contact_data['contact_type'],
                    'standing': contact_data['standing'],
                    'is_watched': contact_data['is_watched']
                }
            )

            contact.labels.clear()
            if contact_data['label_ids'] is not None:
                contact.labels.set([labels[label_id] for label_id in contact_data['label_ids']])

            contacts_to_load.append(contact.pk)

        corporation_token.last_update = timezone.now()
        corporation_token.save()

    group(load_corporation_contact_name.si(pk) for pk in contacts_to_load).delay()


@shared_task
def update_all_alliances_contacts():
    group(
        update_alliance_contacts.si(alliance_token.alliance.alliance_id)
        for alliance_token in AllianceToken.objects.with_valid_tokens().select_related('alliance')
    ).delay()


@shared_task
def update_all_corporations_contacts():
    group(
        update_corporation_contacts.si(corporation_token.corporation.corporation_id)
        for corporation_token in CorporationToken.objects.with_valid_tokens().select_related('corporation')
    ).delay()


@shared_task
def update_all_contacts():
    update_all_alliances_contacts.delay()
    update_all_corporations_contacts.delay()
